//Bouncer Demo for PiTrex/Vectrex Testing

#define PITREX 1

#include <unistd.h>
#include <stdio.h>

char timerreset = 0; //Reset simulated timer (only used for PiTrex)

#ifdef PITREX

#include "pitrexio-lpt.h"

#define VIA_DDR_b            0xD002   //VIA port B data direction register (0=input 1=output)
#define VIA_DDR_a            0xD003   //VIA port A data direction register (0=input 1=output)
#define VIA_t1_cnt_lo    0xD004 // VIA timer 1 count register lo (scale factor)
#define dp_VIA_t1_cnt_lo 0xD004 // VIA timer 1 count register lo (scale factor)
#define VIA_cntl         0xD00C // VIA control register
#define dp_VIA_port_a    0xD001 // VIA port A data I/O register (handshaking)
#define dp_VIA_port_b    0xD000 // VIA port B data I/O register
#define dp_VIA_t1_cnt_hi 0xD005 // VIA timer 1 count register hi
#define dp_VIA_shift_reg 0xD00A // VIA shift register
#define dp_VIA_cntl      0xD00C // VIA control register
#define dp_VIA_int_flags 0xD00D // VIA interrupt flags register

// Added for wait_recal: ADD VIA_t2_cnt_lo VIA_t2_cnt_h
#define dp_VIA_t2        0xD008 // VIA timer 2 count register BOTH BYTES
#define dp_VIA_t2_cnt_lo 0xD008 // VIA timer 2 count register lo
#define dp_VIA_t2_cnt_hi 0xD009 // VIA timer 2 count register hi

#define SETW(what, val) do {vectrexwrite(what, (val)>>8); vectrexwrite((what)+1, (val)&255);} while(0)
#define SET(what, val) vectrexwrite(what, val)
#define GET(what)     vectrexread(what)

int timervalue = 0; //Global Variable for simulated timer upper byte

int vectrexread (int what)
{
 if (timerreset)
 {
  timervalue = 0;
  timerreset = 0;
 }

 if (what == dp_VIA_int_flags)
 {
//  usleep (154); //Equivalent to one timer lower byte cycle in Vectrex. -- Takes ages, also tried usleep (1) and no usleep at all.
  timervalue++; //Equivalent to timer upper byte
 }
 return timervalue;
}

void cDelay (unsigned int units)
{ 
 usleep (units);
}

#else // VECTREX

extern volatile unsigned int VIA_t1_cnt_lo __attribute__ ((section
(".dpd0")));
extern volatile unsigned int VIA_cntl __attribute__ ((section (".dpd0")));
extern volatile int dp_VIA_port_a __attribute__ ((section ("direct")));
extern volatile int dp_VIA_port_b __attribute__ ((section ("direct")));
extern volatile unsigned int dp_VIA_t1_cnt_hi __attribute__ ((section
("direct")));
extern volatile unsigned int dp_VIA_t1_cnt_lo __attribute__ ((section
("direct")));
extern volatile unsigned int dp_VIA_shift_reg __attribute__ ((section
("direct")));
extern volatile unsigned int dp_VIA_cntl __attribute__ ((section
("direct")));
extern volatile unsigned int dp_VIA_int_flags __attribute__ ((section
("direct")));
extern volatile unsigned int dp_VIA_t2_lo __attribute__ ((section
("direct")));
extern volatile unsigned int dp_VIA_t2_hi __attribute__ ((section
("direct")));
extern volatile unsigned long int dp_VIA_t2 __attribute__ ((section
("direct")));

#define SETW(what, val) what = val
#define SET(what, val) what = val
#define GET(what) what

void cDelay (unsigned int units)
{
   (void) units;
}

#endif

static long unsigned int Vec_Rfrsh = 50U; // PLACEHOLDER FOR NOW -- FIX!
static int Vec_Loop_Count = 0;
#define MAX_BRIGHTNESS 255U
static unsigned int Vec_Brightness = MAX_BRIGHTNESS;


const signed char BouncerTitle[7][17] = // count, moveto, drawtos...
{
   {+0x05, +0x67, -0x5A, -0x2E, -0x05, +0x11, +0x1C, +0x08, -0x0D, +0x0E,
    +0x09, +0x07, -0x13,},
   {+0x06, +0x51, -0x38, -0x0D, -0x0C, -0x0E, +0x04, +0x03, +0x11, +0x0C,
    +0x07, +0x0F, -0x05, -0x02, -0x0A,},
   {+0x03, +0x57, -0x1F, -0x18, -0x09, -0x09, +0x13, +0x19, +0x06,},
   {+0x04, +0x56, -0x05, -0x1F, -0x05, +0x1F, +0x0A, +0x01, +0x0D, -0x20,
    +0x00,},
   {+0x04, +0x59, +0x26, -0x03, -0x0F, -0x11, -0x04, -0x0E, +0x03, +0x04,
    +0x13,},
   {+0x07, +0x4D, +0x38, +0x04, +0x0D, +0x06, +0x03, +0x03, -0x0F, -0x05,
    -0x0A, -0x0F, -0x02, -0x0E, +0x03, +0x02, +0x15,},
   {+0x04, +0x5A, +0x50, -0x04, +0x03, -0x1E, -0x03, +0x20, +0x06, +0x02,
    +0x0F,},
};

static void cMov_Draw_VLc_a (signed char *vList)
{
   register int count = *(vList) - 1; // count in list

   SET (dp_VIA_port_a, *(1 + vList)); // y pos to dac
   SET (dp_VIA_cntl, 0xce); // disable zero, disable all blank
   SET (dp_VIA_port_b, 0); // mux enable, dac to -> integrator y (and x)
   SET (dp_VIA_port_b, 1); // mux disable, dac only to x
   SET (dp_VIA_port_a, *(2 + vList)); // dac -> x
   SET (dp_VIA_t1_cnt_hi, 0); // start timer
   vList += 3;

   timerreset = 1;
   while ((GET (dp_VIA_int_flags) & 0x40) == 0) ; // wait till timer finishes
   // This will have to be modified until you get reading to work...

   do {
      SET (dp_VIA_port_a, *(vList)); // first y coordinate to dac
      SET (dp_VIA_port_b, 0); // mux enable, dac to -> integrator y (and x)
      SET (dp_VIA_port_b, 1); // mux disable, dac only to x
      SET (dp_VIA_port_a, *(vList + 1)); // dac -> x
      SET (dp_VIA_shift_reg, 0xff); // full "unblank" output
      SET (dp_VIA_t1_cnt_hi, 0); // start timer
      vList += 2;

      timerreset = 1;
      while ((GET (dp_VIA_int_flags) & 0x40) == 0) ; // wait till timer finishes
      // This will have to be modified until you get reading to work...

      SET (dp_VIA_shift_reg, 0); // output full blank
   } while (--count >= 0); // loop thru all vectors
}

static void cReset0Ref ()
{
   SET (dp_VIA_cntl, 0xcc); // enable zero, enable all blank

   // reset integrators
   SET (dp_VIA_port_a, 0); // dac = 0
   SET (dp_VIA_port_b, 3); // mux=1, disable mux
   SET (dp_VIA_port_b, 2); // mux=1, enable mux
   SET (dp_VIA_port_b, 2); // delay
   SET (dp_VIA_port_b, 1); // disable mux
}

static void cMoveto_d (signed int y, signed int x)
{
   SET (dp_VIA_port_a, y); // y pos to dac
   SET (dp_VIA_cntl, 0xce); // disable zero, disable all blank
   SET (dp_VIA_port_b, 0); // mux enable, dac to -> integrator y (and x)
   SET (dp_VIA_shift_reg, 0); // all output is BLANK
   SET (dp_VIA_port_b, 1); // mux disable, dac only to x
   SET (dp_VIA_port_a, x); // dac -> x
   SET (dp_VIA_t1_cnt_hi, 0); // start timer
   timerreset = 1;
   while ((GET (dp_VIA_int_flags) & 0x40) == 0) ; // wait till timer finishes
}

static void cWait_Recal ()
{
   Vec_Loop_Count++;
   timerreset = 1;
   while ((GET (dp_VIA_int_flags) & 0x20) == 0) ; // for 50 Hz

   SETW (dp_VIA_t2, Vec_Rfrsh);
   SET (dp_VIA_t1_cnt_lo, (unsigned int) 255);
   cMoveto_d (127, 127);
   SET (dp_VIA_cntl, 0xcc); // enable zero, enable all blank
   cMoveto_d (-128, -128);

   cReset0Ref();
}

static void cIntensity_a (unsigned int i)
{
   SET (Vec_Brightness, i);
   SET (dp_VIA_port_a, (int) i); // Store intensity in D/A
   SET (dp_VIA_port_b, 5); // mux disabled channel 2
   SET (dp_VIA_port_b, 4); // mux enable
   SET (dp_VIA_port_b, 1); // turn off mux
}

int main (void)
{
#ifdef PITREX
   if (!vectrexinit() ) //Initialise Vectrex Connection
   {
    printf("Could Not Initialise Vectrex Connection\n");
    return 1;
    
    //Configure Peripheral Port Inputs/Outputs
    vectrexwrite (VIA_DDR_b, 0x9F); // All Outputs except COMPARE input and PB6 at Cartridge Port
    vectrexwrite (VIA_DDR_a, 0xFF); // All Outputs
   }
#endif

   for (;;) { /* start vectrex round */

      printf("cycle\n");
      cWait_Recal ();
      SET (VIA_t1_cnt_lo, 0x80);
      cIntensity_a (MAX_BRIGHTNESS / 2); /* set intensity of vector beam... */
      int i = 0;
      for (; i < 7; i++) {
cDelay (1);
SET (VIA_cntl, 0xCE);
cMov_Draw_VLc_a ((void *) BouncerTitle[i]);
SET (VIA_cntl, 0xCC);
      }
   }
}
