// PiTrex Discrete LPT Write Test Software.
// -Draws an image on Vectrex display via PC Parallel port.
// Kevin Koster, 2018.

// PiTrex Discrete Read (TODO) & Write Functions, via PC Parallel Port

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <linux/parport.h>
#include <linux/ppdev.h>

/*
  Port Assignments:
  ^^^^^^^^^^^^^^^^^
  Outputs:
  Data 0-7: Data 0-7 & A0, A1, A2, A3, A12, A14/A15
  Control #STROBE (pin 1)[inv.]: LATCH EN
  Control #AUTOFD (pin 14)[inv.]: #D/A
  Control #SELIN (pin 17)[inv.]: R/#W
  
  Inputs:
  Status ACK (pin 10): RDY
  Status PE (pin 12): #HALT
  Status SEL (pin 13): IRQ
*/

int lpt;

// Initialise parallel port for communication with the Vectrex
int vectrexinit ()
{
	unsigned char controlalloff = PARPORT_CONTROL_STROBE | PARPORT_CONTROL_SELECT | PARPORT_CONTROL_AUTOFD; // Outputs inverted
	char *portpath = "/dev/parport0"; // Command-line argument?
	int result;
	
	lpt = open(portpath, O_RDWR);
	result = ioctl(lpt,PPCLAIM);
	
	if (result >= 0)
	 result = ioctl (lpt, PPWCONTROL, &controlalloff); //LATCH EN Low  R/#W Low  #D/A Low

	if (result >= 0)
	 result = lpt;

//	printf("init LPT= %d\n", lpt);
	 
	return result;
}

// Write a byte to the Vectrex at the specified address
// --TODO: select only wanted bits of full address.
void vectrexwrite (unsigned int address, unsigned char data)
{
	int dir = 0; //Output
	unsigned char status;
	unsigned char controlalloff = PARPORT_CONTROL_STROBE | PARPORT_CONTROL_SELECT | PARPORT_CONTROL_AUTOFD; // Outputs inverted
	unsigned char controlautofd = PARPORT_CONTROL_STROBE | PARPORT_CONTROL_SELECT;
	unsigned char controlstrobe = PARPORT_CONTROL_SELECT | PARPORT_CONTROL_AUTOFD;
	unsigned char controlautofdstrobe = PARPORT_CONTROL_SELECT;
	
	unsigned char rdy = PARPORT_STATUS_ACK;
	
	//Sets highest 2 bits permanently High as all the addresses have that, ORed with lowest four bits of address. -- Cheated
	unsigned char shortaddress = (address & 0x0F) | 0x30;
	
//	printf("run LPT= %d\n", lpt);
	
	ioctl (lpt, PPWCONTROL, &controlalloff); //LATCH EN Low  R/#W Low  #D/A Low
	ioctl (lpt, PPDATADIR, &dir); // Data Output Mode
	ioctl (lpt, PPWDATA, &data); // Data on Data Bus
	do
	{
	 ioctl (lpt, PPRSTATUS, &status);
	}
	while (!(status & rdy)); // Loop until RDY goes High
	
	printf("whoo-hoo, RDY went HIGH\n");
	
	ioctl (lpt, PPWCONTROL, &controlautofd); // #D/A High
	ioctl (lpt, PPWDATA, &shortaddress); // Address on Address Bus
//	usleep(100);
	ioctl (lpt, PPWCONTROL, &controlautofdstrobe); //LATCH EN High
//	usleep (100);
//	ioctl (lpt, PPWCONTROL, &controlstrobe); //LATCH EN High, #D/A Low.
	do
	{
	 ioctl (lpt, PPRSTATUS, &status);
	}
	while (status & rdy); // Loop until RDY goes Low
	
	do
	{
	 ioctl (lpt, PPRSTATUS, &status);
	}
	while (!(status & rdy)); // Loop until RDY goes High
	usleep(1000000);
	printf("end\n");
	ioctl (lpt, PPWCONTROL, &controlalloff); //LATCH EN Low  R/#W Low  #D/A Low
}

// Wait until interrupt from Vectrex
void vectrexinterrupt (char interrupt)
{
	unsigned char irq = PARPORT_STATUS_SELECT;
	unsigned char status;
	unsigned char rdy = PARPORT_STATUS_ACK;
	
	if (interrupt)
	{
	 do
	 {
	  ioctl (lpt, PPRSTATUS, &status);
	 }
	 while (!(status & rdy)); // Loop until RDY goes High
	}

	else
	{
	 do
	 {
	  ioctl (lpt, PPRSTATUS, &status);
	 }
	 while (status & rdy); // Loop until RDY goes Low
	}
}

int vectrexclose ()
{
	int result = ioctl(lpt,PPRELEASE);
	result = close(lpt);

	return result;
}
